#! /bin/csh -f
#  $Id: quicklook.sh,v 1.26 2004/06/09 23:16:47 jhf Exp $
#  Generate simple quicklook JPEG for TDF or EOS HDF
#  Input file usage  is quicklook [parameter=...] input-file output-file
#  Parameters are
#	satellite   - required if input is HDF, and if no pass number is supplied
#	variable    - required if input is a dataset
#	remap       - optional remap choice; default is yes
#	master      - optional master file for remap; default is auto generated
#	scale_range - optional linear scale range; default is non-linear scaling
#	invert      - optional invert scale; default is no
#	enhancement - optional enhancement table; default is histogram equalization
#	palette     - optional color palette; default is black-white
#	overlays    - optional yes/no geopolitical overlays and legend
#	grid_delta  - optional lat/lon grid spacing in degrees, default is automatic
#	boundaries  - coastline databases, default is dcw.coast dcw.political
#	legend      - optional yes/no legend overlay
#	format      - optional output image format; default is jpeg
#  overlays=no overrides legend=yes and all other overlay options
#  lat/lon grid overlay can be turned of by setting grid_delta = 0
#  scale_range overrides enhancement

set FUNC = `basename $0`
set OUTFILE = ()
unsetenv TMPDIR
onintr gotsig

echo ""
echo ${FUNC}: Started at `date`

source $TSCANROOT/bin/tsargs
source $TSCANROOT/bin/logrundefs

# Check input and output files
# ----------------------------

set OUTFILE = (`tsparam -f $argv:q`)
if ($#OUTFILE != 2) then
  shout ""
  shout "Usage: ${FUNC} [parameter=...] input-file output-file"
  goto failed
endif

if ($#OUTFILE == 2) then
  set INPFILE = ( $OUTFILE[1] )
  shift OUTFILE
  set PASS = ( 0 )
  if ($?LASTPASS == 0) then
    setenv LASTPASS $PASS
  endif
endif

# Get command line parameters
# ---------------------------

set USERVAR  = (`tsparam -p variable -d ""  $argv:q`)
set REMAP    = (`tsparam -p remap    -d yes $argv:q`)
set USERMAP  = (`tsparam -p master   -d ""  $argv:q`)
set USERPAL  = (`tsparam -p palette  -d ""  $argv:q`)
set USERENH  = (`tsparam -p enhancement -d "" $argv:q`)
set RANGE    = (`tsparam -p scale_range -d "" $argv:q`)
set INVERT   = (`tsparam -p invert   -d no  $argv:q`)
set IMGFMT   = (`tsparam -p format  -d jpeg $argv:q`)
set OVERLAYS = (`tsparam -p overlays -d yes $argv:q`)
set FILTER   = (`tsparam -p filter   -d yes $argv:q`)
set LEGEND   = (`tsparam -p legend   -d no  $argv:q`)
set GRID     = (`tsparam -p grid_delta  -d -1  $argv:q`)
set COAST    = (`tsparam -p boundaries  -d "dcw.coast dcw.political"  $argv:q`)
set SATEL    = (`tsparam -p satellite -d "" $argv:q`)
set MAXWIDTH = (`tsparam -p max_width -d 0 $argv:q`)
set MAXHEIGHT = (`tsparam -p max_height -d 0 $argv:q`)

if ($#RANGE != 0 && $#RANGE != 2) then
  echo ""
  echo ${FUNC}: Exactly two numbers required for range
  goto failed
endif

# Refine yes/no parameters
# ------------------------

echo $REMAP | grep '^[yY]' > /dev/null
if ($status == 0) set REMAP = yes

echo $OVERLAYS | grep '^[yY]' > /dev/null
if ($status == 0) set OVERLAYS = yes

echo $FILTER | grep '^[yY]' > /dev/null
if ($status == 0) set FILTER = yes

echo $LEGEND | grep '^[yY]' > /dev/null
if ($status == 0) set LEGEND = yes

echo $INVERT | grep '^[yY]' > /dev/null
if ($status == 0) then
  set INVERT = yes
else
  set INVERT = no
endif

# Variable is required for dataset input
# --------------------------------------

if ($#INPFILE == 1) then
  if ($#USERVAR == 0) then
    echo ""
    echo ${FUNC}: Variable required for input file
    goto failed
  endif
  set VAR = $USERVAR
endif

# Check to see if master file exists
# ----------------------------------

if ($#USERMAP == 1) then
  if (! -f $USERMAP) then
    set CHECK = $PASSDIR/masters/$USERMAP
    if (-f $CHECK) then
      set USERMAP = $CHECK
    else
      echo ${FUNC}: Cannot find master file $USERMAP
      goto failed
    endif
  endif
endif

# Create temporary working directory
# ----------------------------------

set TMPDIR = $PGSWORKDIR/qlook.$$

/bin/rm -rf $TMPDIR

logrun - mkdir $TMPDIR

if ($status != 0) goto failed

set INGFILE = $TMPDIR/Ingest
set REGFILE = $TMPDIR/Fastreg
set IMGFILE = $TMPDIR/Image
set TMPFILE = $TMPDIR/Tmp
set FILTFILE = $TMPDIR/Filter
set HISTFILE = $TMPDIR/Bvhist
set MASTER  = $TMPDIR/Master
set PICFILE = $TMPDIR/Picture

/bin/rm -f $OUTFILE

# Process input from dataset
# --------------------------

set ENHANCE = ()
set PALETTE = ()

if (! -f $INPFILE) then
    echo ""
    echo ${FUNC}: No such input file $INPFILE
    goto failed
endif

# Fully qualify input file path so links work
# -------------------------------------------

set SLASH = `echo $INPFILE | cut -c 1-1`
if ("$SLASH" != "/") then
    set INPFILE = `pwd`/$INPFILE
endif

# Need to instantiate HDF link for other TeraScan functions to work
# -----------------------------------------------------------------

if ("$INPFILE:e" == "hdf") then
    if ($#SATEL == 0) then
      shout ""
      shout ${FUNC}: Need satellite to import EOS HDF
      goto failed
    endif

    logrun - link_hdf_eos satellite=$SATEL create=yes $INPFILE $INGFILE
    if ($status != 0) goto failed
    logrun - subset include_vars=$VAR clip_dims= instantiate=no $INGFILE $TMPFILE
    if ($status != 0) goto failed
    logrun - hinstant $TMPFILE $INGFILE
    if ($status != 0) goto failed
    \rm -f $TMPFILE
    set FORMAT = HDF
else
    softlink $INPFILE $INGFILE
    set FORMAT = TDF
endif

# Make sure user specified variable is in dataset
# -----------------------------------------------

set CHECK = `varnames include_vars=$VAR $INGFILE`
if ($#CHECK == 0) then
    echo ""
    echo ${FUNC}: Cannot find variable $VAR in $INPFILE
    goto failed
endif

# If LASTPASS not supplied, get pass info from dataset
# ----------------------------------------------------

if ($LASTPASS == 0) then
    logset SATELLITE getinfo attr_name=satellite $INGFILE
    if ($status != 0) goto failed
    logset SENSOR getinfo attr_name=sensor_name $INGFILE
    if ($status != 0) goto failed
    logset PASS_DATE getinfo attr_name=pass_date $INGFILE
    if ($status != 0) goto failed
    logset PASS_TIME getinfo attr_name=start_time $INGFILE
    if ($status != 0) goto failed

    # Get telemetry for satellite/sensor
    # ----------------------------------

    set CHECK = `grep "\.sensors:.*$SENSOR" $SATDATA/$SATELLITE/Define`
    if ($status == 0) then
      set TELEMETRY = `echo $CHECK | sed -e 's/\.sensors:.*$//'`
    else
      set TELEMETRY = unknown
    endif

    echo ""
    echo ${FUNC}: Telemetry for $SATELLITE $SENSOR is $TELEMETRY

endif

# Decide where dataset processing needs to start
# ----------------------------------------------

softlink $INGFILE $FILTFILE

if ($REMAP == yes && $#USERMAP == 1) then
    logrun - cp $USERMAP $MASTER
    if ($status != 0) goto failed
    set AUTO_MASTER = no
    set MUST_REMAP  = yes
else

    logset PROJ getinfo attr_name=projection_name $INGFILE
    if ($status != 0) set PROJ = none

    if ($REMAP != yes || $PROJ != sensor_scan) then
      softlink $INGFILE $REGFILE
      set AUTO_MASTER = no
      set MUST_REMAP  = no
    else
      set AUTO_MASTER = yes
      set MUST_REMAP  = yes
    endif
endif

# Automatic master file generation
# Determine latitude/longitude extents
# ------------------------------------

logset EXTENTS etxinfo std_format=no real_output=yes item=extents $FILTFILE

if ($status != 0) goto failed

if ($#EXTENTS != 4) then
  shout ""
  shout ${FUNC}: Failed to compute lat/lon extents
  goto failed
endif

echo $EXTENTS |& grep unknown > /dev/null
if ($status == 0) then
  shout ""
  shout ${FUNC}: Failed to compute lat/lon extents
  goto failed
endif

set LONRANGE = `smath "nint(mod(720. + $EXTENTS[4] - $EXTENTS[3], 360.))"`

# Create master based on extents
# ------------------------------

if ($AUTO_MASTER == yes) then

  set MINLAT = `smath "nint($EXTENTS[1])"`
  set MAXLAT = `smath "nint($EXTENTS[2])"`

  set WIDTH = 1024
  if ($MAXWIDTH > 0) set WIDTH = $MAXWIDTH
  set HEIGHT = 1024
  if ($MAXHEIGHT > 0) set HEIGHT = $MAXHEIGHT

  if ($MAXLAT < 85 && $MINLAT > -85) then
    logrun - master2 \
	projection=rect		\
	min_lat=$EXTENTS[1]	\
	max_lat=$EXTENTS[2]	\
	min_lon=$EXTENTS[3]	\
	max_lon=$EXTENTS[4]	\
	corner_edges=yes	\
	square_aspect=yes	\
	num_samples=$WIDTH	\
	est_center=yes		\
	$MASTER
    if ($status != 0) goto failed
  else
    logset MIDLAT etxinfo item=center_lat real_output=yes std_format=no $FILTFILE
    if ($status != 0) goto failed

    logset MIDLON etxinfo item=center_lon real_output=yes std_format=no $FILTFILE
    if ($status != 0) goto failed

    if ($MIDLAT == unknown || $MIDLON == unknown) then
      shout ""
      shout ${FUNC}: Cannot compute center lat/lon
      goto failed
    endif

    logrun - master		\
	projection=stereo	\
	center_lat=$MIDLAT	\
	center_lon=$MIDLON	\
	num_lines=$HEIGHT	\
	num_samples=$WIDTH	\
	pixel_width=6		\
	pixel_height=6		\
	rotate_angle=0		\
	move_center=no		\
	$MASTER
    if ($status != 0) goto failed
  endif

endif

# Remap ingested dataset to master
# --------------------------------

if ($MUST_REMAP == yes) then

  logrun - fastreg2		\
    sensor_resol=no		\
    master_file=$MASTER		\
    master_var=			\
    include_vars=$VAR		\
    add_to_output=no		\
    poly_size=100		\
    interpolate=ngn		\
    off_protect=yes		\
    brute_force=no		\
    whole_input=no		\
    max_memory=32		\
    $FILTFILE $REGFILE

  if ($status != 0) goto failed

endif

after_reg:

# Check the palette file
# ----------------------

if ($#USERPAL == 1) then
  set PALETTE = $USERPAL
endif

if ($#PALETTE == 1) then
  if (! -f $PALETTE) then
    set CHECK = $REFDATA/palettes/$PALETTE
    if (-f $CHECK) then
      set PALETTE = $CHECK
    else
      echo ${FUNC}: Cannot find palette file $CHECK
      set PALETTE = black-white
    endif
  endif
else
  set PALETTE = black-white
endif

# Check the enhancement file
# --------------------------

if ($#USERENH == 1) then
  set ENHANCE = $USERENH
endif

if ($#ENHANCE == 1) then
  if (! -f $ENHANCE) then
    set CHECK = $REFDATA/enhancements/$ENHANCE
    if (-f $CHECK) then
      set ENHANCE = $CHECK
    else
      echo ${FUNC}: Cannot find enhancement file $CHECK
      set ENHANCE = ()
    endif
  endif
endif

# Reduce size if bigger than configuration
# Only needed if imscale not being run
# ----------------------------------------

set MAXSZ = 64000
set WIDTH = $MAXSZ
if ($MAXWIDTH > 0) set WIDTH = $MAXWIDTH
set HEIGHT = $MAXSZ
if ($MAXHEIGHT > 0) set HEIGHT = $MAXHEIGHT

logset SIZE varinfo var_name=$VAR attr_name=size $REGFILE
if ($status != 0) goto failed

if ($#SIZE != 2) then
  shout ""
  shout ${FUNC}: Variable $VAR is not an image
  goto failed
endif

if ( $#RANGE != 2 && ( $SIZE[1] > $HEIGHT || $SIZE[2] > $WIDTH ) ) then
  set HSCALE = `smath "$HEIGHT/$SIZE[1]"`
  set WSCALE = `smath "$WIDTH /$SIZE[2]"`
  set SCALE  = `smath "($HSCALE < $WSCALE) ? $HSCALE : $WSCALE"`

  logrun - magnify include_vars=$VAR method=nn \
	use_scale=yes y_scale=$SCALE x_scale=$SCALE \
	$REGFILE $TMPFILE
  if ($status != 0) goto failed

  \rm -f $REGFILE
  logrun - /bin/mv -f $TMPFILE $REGFILE
endif

# Byte scale the remapped data
# ----------------------------

set IMCOLORS = 254

if ($#RANGE == 2) then
    set CHECK = `smath "$RANGE[1] > $RANGE[2]"`
    if ("$CHECK" == "1") then
        set INVERT = yes
        set RANGE = ( $RANGE[2] $RANGE[1] )
    endif
    logrun - imscale		\
	include_vars=$VAR	\
	image_colors=$IMCOLORS	\
	max_width=$WIDTH	\
	max_height=$HEIGHT	\
	fixed_size=no		\
	zoom_factor=1		\
	real_resample=yes	\
	est_range=no		\
	min_value=$RANGE[1]	\
	max_value=$RANGE[2]	\
	invert_scale=$INVERT	\
	bad_color=255		\
    $REGFILE $IMGFILE
    if ($status != 0) goto failed

else if ($#ENHANCE == 1) then
    logrun - enhance		\
	include_vars=$VAR	\
	lookup_file=$ENHANCE	\
        value_var=value		\
	index_var=index		\
	image_colors=$IMCOLORS	\
        bad_index=255		\
    $REGFILE $IMGFILE
    if ($status != 0) goto failed
else
    logrun - bvhist		\
	hist_vars=$VAR		\
	bin_counts=$IMCOLORS	\
        estimate_range=yes	\
        count_outside=yes	\
	log_scale=no		\
	unit_scale=no		\
        lookup_table=yes	\
	bin_centers=		\
    $REGFILE $HISTFILE
    if ($status != 0) goto failed

    if ($INVERT == yes) then
      logrun - emath			\
	expr_vars=histogram		\
	expression="$IMCOLORS - x1"	\
	var_name=histogram		\
	var_units=			\
	var_type=byte			\
	offset=0			\
	scale_factor=1			\
	$HISTFILE $IMGFILE
      if ($status != 0) goto failed

      logrun - copyvar		\
	include_vars=histogram	\
	overwrite_vars=yes	\
	$IMGFILE $HISTFILE
      if ($status != 0) goto failed

      /bin/rm -f $IMGFILE
    endif

    logrun - enhance		\
	include_vars=$VAR	\
	lookup_file=$HISTFILE	\
        value_var=$VAR		\
	index_var=histogram	\
	image_colors=$IMCOLORS	\
        bad_index=255		\
    $REGFILE $IMGFILE
    if ($status != 0) goto failed

    /bin/rm -f $HISTFILE
endif

/bin/rm -f $REGFILE

# Inject overlays
# No overlays possible for wefax, even legend
# -------------------------------------------

if ($OVERLAYS == yes && $TELEMETRY != wefax) then

    if ($#COAST > 0) then
      logrun - coast		\
	master_file=$IMGFILE	\
	reduce_factor=2		\
	coast_file="$COAST"	\
      $TMPDIR/Coast
      if ($status != 0) goto failed
    endif

    set COLORS_PER = 2
    set METAFILES = ( $TMPDIR/Coast )
    set METACOLORS = ( $IMCOLORS $IMCOLORS )

    if ($GRID) then
      if ($GRID > 0) then
        set DELTA = $GRID
      else if ($LONRANGE > 90) then
        set DELTA = 10
      else
        set DELTA = 5
      endif

      logrun - llgrid		\
	master_file=$IMGFILE	\
	grid_type=lines		\
	delta_lat=$DELTA	\
	delta_lon=$DELTA	\
      $TMPDIR/Grid
      if ($status != 0) goto failed

      set METAFILES = ( $METAFILES $TMPDIR/Grid )
      set METACOLORS = ( $METACOLORS $IMCOLORS $IMCOLORS )
    endif

    if ($LEGEND == yes) then
	set SATEL = `echo $SATELLITE | tr '[a-z]' '[A-Z]'`
        set SENSOR = `echo $VAR | tr '[a-z]' '[A-Z]'`
	logrun - legend	\
		line_1="$SATEL $SENSOR $PASS_DATE $PASS_TIME"		\
		line_2=			\
		text_height=18		\
		text_type=plain		\
		line_offset=0		\
		sample_offset=0		\
		full_width=yes		\
		center_text=yes		\
		outline=no		\
		solid_bg=yes		\
	$IMGFILE $TMPDIR/Legend
	if ($status != 0) goto failed

	set METAFILES = ( $METAFILES $TMPDIR/Legend )
	set METACOLORS = ( $METACOLORS $IMCOLORS 255 )
    endif

    logrun - sinject		\
	bg_var_name=$VAR	\
	colors_per=$COLORS_PER	\
	meta_colors="$METACOLORS"	\
	line_widths=		\
	line_types=		\
	marker_types=		\
	marker_sizes=		\
	image_colors=$IMCOLORS	\
    $IMGFILE $METAFILES $PICFILE
    if ($status != 0) goto failed

    /bin/rm -f $TMPDIR/Coast $TMPDIR/Grid $TMPDIR/Legend $IMGFILE
else
  set PICFILE = $IMGFILE
endif

# Export the byte scaled data
# ---------------------------

if ($IMGFMT == tdf) then
  logrun - /bin/cp -f $PICFILE $OUTFILE
  if ($status != 0) goto failed
else
  logrun - expim \
	image_format=$IMGFMT	\
	image_var=$VAR		\
	image_colors=$IMCOLORS	\
	color_palette=$PALETTE	\
	draw_indexes="$IMCOLORS 255"	\
	draw_names="black white"	\
  $PICFILE $OUTFILE

  if ($status != 0) goto failed
endif

# Termination section
# -------------------

set STATUS = 0
goto done

gotsig:

shout ""
shout ${FUNC}: Failed - Received interrupt signal

failed:

set STATUS = 1

if ($#OUTFILE == 1) then
  /bin/rm -f $OUTFILE
endif

done:

if ($?TMPDIR) then
 /bin/rm -rf $TMPDIR
endif

echo ""
echo ${FUNC}: Ended at `date`
echo ${FUNC}: Exiting with error status $STATUS

exit $STATUS
